--[[
A generic button class

This code is MIT licensed, see http://www.opensource.org/licenses/mit-license.php
(C) 2010 - 2011 Gideros Mobile 


BW (2012)
  - Modified to remove down state image and instead darken the button when pressed
  - Modified to add function headers to help my understanding of the code
  - Modified to stop improper button over effects when using glider
]]

Button = gideros.class(Sprite)

--=================================================================================================
-- Function : init
-- Purpose  : initializes a new button
-- Params   : image - image to use for the new button
-- Return   : n/a
--=================================================================================================
function Button:init(image)

	self.btnImage = image  -- add the sprite as the buttons image	
	self:addChild(self.btnImage)  -- add as button scene
		
	self.focus = false
	self:updateVisualState(false)

	self:addEventListener(Event.MOUSE_DOWN, self.onMouseDown, self)
	self:addEventListener(Event.MOUSE_MOVE, self.onMouseMove, self)
	self:addEventListener(Event.MOUSE_UP, self.onMouseUp, self)

	self:addEventListener(Event.TOUCHES_BEGIN, self.onTouchesBegin, self)
	self:addEventListener(Event.TOUCHES_MOVE, self.onTouchesMove, self)
	self:addEventListener(Event.TOUCHES_END, self.onTouchesEnd, self)
	self:addEventListener(Event.TOUCHES_CANCEL, self.onTouchesCancel, self)
end

--=================================================================================================
-- Function : onMouseDown
-- Purpose  : called on mouse down events that occue on the button
-- Params   : event - the touch event that fired this callback
-- Return   : n/a
--=================================================================================================
function Button:onMouseDown(event)
	if self:hitTestPoint(event.x, event.y) then
		self.focus = true
		self:updateVisualState(true)
		event:stopPropagation()
	end
end

--=================================================================================================
-- Function : onMouseMove
-- Purpose  : called on mouse move events that occur on the button
-- Params   : event - the touch event that fired this callback
-- Return   : n/a
--=================================================================================================
function Button:onMouseMove(event)
	if self.focus then
		if not self:hitTestPoint(event.x, event.y) then
			self.focus = false;
			self:updateVisualState(false)
		end
		event:stopPropagation()
	end
end

--=================================================================================================
-- Function : onMouseUp
-- Purpose  : called on mouse up events that occur on the button
-- Params   : event - the touch event that fired this callback
-- Return   : n/a
--=================================================================================================
function Button:onMouseUp(event)
	if self.focus then
		self.focus = false;
		self:updateVisualState(false)
		self:dispatchEvent(Event.new("click"))
		event:stopPropagation()
	end
end

--=================================================================================================
-- Function : onTouchesBegin
-- Purpose  : if button is on focus, stop propagation of touch events
-- Params   : event - the touch event that fired this callback
-- Return   : n/a
--=================================================================================================
function Button:onTouchesBegin(event)
	if self.focus then
		event:stopPropagation()
	end
end

--=================================================================================================
-- Function : onTouchesMove
-- Purpose  : if button is on focus, stop propagation of touch events
-- Params   : event - the touch event that fired this callback
-- Return   : n/a
--=================================================================================================
function Button:onTouchesMove(event)
	if self.focus then
		event:stopPropagation()
	end
end

--=================================================================================================
-- Function : onTouchesEnd
-- Purpose  : if button is on focus, stop propagation of touch events
-- Params   : event - the touch event that fired this callback
-- Return   : n/a
--=================================================================================================
function Button:onTouchesEnd(event)
	if self.focus then
		event:stopPropagation()
	end
end

--=================================================================================================
-- Function : onTouchesCancel
-- Purpose  : if touches are cancelled, reset the state of the button
-- Params   : event - the touch event that fired this callback
-- Return   : n/a
--=================================================================================================
function Button:onTouchesCancel(event)
	if self.focus then
		self.focus = false;
		self:updateVisualState(false)
		event:stopPropagation()
	end
end

--=================================================================================================
-- Function : updateVisualState
-- Purpose  : if state is true show downState else show upState
-- Params   : state - state of the button (pressed = true / false)
-- Return   : n/a
--=================================================================================================
function Button:updateVisualState(state)
	if state then
	    -- make the button appear pressed
		self.btnImage:setColorTransform(0.65, 0.65, 0.65, 1)
	else
	    -- make the button appear unpressed
		self.btnImage:setColorTransform(1, 1, 1, 1)
	end
end
